/*  xb_dumpdbt.cpp

XBase64 Software Library

Copyright (c) 1997,2003,2014,2017 Gary A Kunkel

The xb64 software library is covered under 
the terms of the GPL Version 3, 2007 license.

Email Contact:

    xb64-devel@lists.sourceforge.net
    xb64-users@lists.sourceforge.net

*/


#include <xbase.h>

using namespace xb;

int main( int ac, char **av )
{


#ifdef XB_MEMO_SUPPORT
  xbXBase x;

  xbInt16  iRc;
  char     cFieldType;
  xbDbf    *MyFile = NULL;
  xbUInt32 ulMemoFieldLen;
  xbString sFldName;
  xbInt32  lBlockPtr;
  xbString sMemoFldData;

  if( ac <= 1 ){
    std::cout << "\nUsage: dumpdbt filename...\n";
    return 1;
  }

  if(( iRc = x.OpenHighestVersion( av[1], "", &MyFile )) != XB_NO_ERROR ){
    std::cout << "Could not open file iRc = " << iRc  << " file = "  << av[1] << std::endl;
    x.DisplayError( iRc );
    return 0;
  }

  if( MyFile->GetMemoFieldCnt() == 0 ) {
    std::cout << "No memo fields exist in " << av[1] << std::endl;
  } else {

    xbUInt32 ulRecCnt = 0;
    iRc = MyFile->GetRecordCnt( ulRecCnt );

    for( xbUInt32 ul = 1; ul <= ulRecCnt; ul++ ){
      MyFile->GetRecord( ul );
      std::cout << "\nRecord # " << MyFile->GetCurRecNo();
      if( MyFile->RecordDeleted())
        std::cout << " (deleted) ";

      for( int j = 0; j < MyFile->GetFieldCnt(); j++ ) {
        MyFile->GetFieldType( j, cFieldType );

        if( cFieldType == 'M' ) {
          MyFile->GetMemoFieldLen( j, ulMemoFieldLen );
          MyFile->GetFieldName( j, sFldName );
          MyFile->GetLongField( j, lBlockPtr );
          std::cout << "\nMemo field [" << sFldName.Str()
              << "] length = [" << ulMemoFieldLen;
          std::cout << "] Head Block = [" << lBlockPtr << "]\n";

          MyFile->GetMemoField( j, sMemoFldData );
          std::cout << sMemoFldData.Str() << "\n";
        }
      }
    }
    std::cout << "\n";
    MyFile->Close();
    delete MyFile;
  }

#else
    std::cout << "\nXB_MEMO_SUPPORT is OFF\n";
#endif

  return 0;
}
